#!/usr/bin/php
<?


#Script to poll the upload queue for the latest file
#and upload it to the server via rsync.

#Just a wrapper, all the heavy lifting is done in transfer.inc.

#Change to the directory we are running from so we properly include other files
chdir( dirname( $_SERVER['SCRIPT_FILENAME'] ) );

include "transfer.inc";
include "error.inc";

$source = ISUMA_ROOT . "/media";
$destination = "cachingserver@isuma.tv:/persistent/www/isumatv/sites/default/files/video-local-servers/";
$queuename = "upload";

// get the lockfile early so that network errors below will not stack up
acquire_lockfile() or trigger_error( "Lockfile creation failed. Held by another process.", E_USER_ERROR );

//Get upload rate limit from server and choose a reasonable default if not present
$configs = getconfig( XMLRPC_SERVER );

$rate = $configs['upload_bandwidth'];

if( !isset( $rate ) || !is_numeric( $rate ) ){
  if( DEBUG ){
    trigger_error( "Download rate limit not specified or not a number. Defailting to 10KB/s.", E_USER_WARNING );
  }
  $rate = 10;
}else{
  if( DEBUG ){
    trigger_error( "Got upload rate limit of " . $rate . "KB/s from server.", E_USER_NOTICE );
  }
}

#delete all files corresponding to unpublished nodes so that we don't
#have any accidental name conflicts
deleteunpublished( XMLRPC_SERVER, $source, "file" );

#now perform the actual transfer
queuetransfer( XMLRPC_SERVER, $source, $destination, $queuename, $rate );

exit_successfully();
