<?php

include( "ffmpeg_wrapper.module" );


/**
 * Fake function to fake out the ffmpeg wrapper scripts
 */

function variable_get( $var, $default ){
  return $default;
}

/**
 * Implementation of media_mover hook
 * @param $op is the operator
 * @param $action_id is the action to call
 * @param $configuration is the media mover configuration currently being run
 * @param $file is the media mover file array
 * 
 */
function mm_isuma_media_mover($op, $action_id = null, $configuration = array(), &$file = array(), $job = null, $nid = null ) {
  switch ($op) {
  	// identify this module
    case 'name':
      return "Isuma module";
    break;

    // define this module's actions   
    case 'actions':
      return array(
        'harvest' => array(
          6 => 'isuma ftp uploads',
          8 => 'isuma livetv'
        ),
        'storage' => array(
          7 => 'isuma ftp node save',
          9 => 'isuma livetv node save',
          10 => 'isuma metadata'
        ),
        'process' => array( 
          1 => 'isuma hi bandwidth flv',
          2 => 'isuma lo bandwidth flv',
          3 => 'isuma thumbnail',
          4 => 'isuma audio',
          5 => 'isuma ipod',
          11 => 'isuma hd flv',
        )
      );
    break;

    // define any directories that might need to be created
    case 'directories':
      return array('isuma');
    break;

    case 'harvest':
      switch ($action_id) {
        case '6': // isuma ftp uploads
          return mm_isuma_ftp_harvest($configuration);
        break;
        case '8': // isuma ftp uploads
          return mm_isuma_livetv_harvest($configuration);
        break;
      }
    break;
    
    case 'storage':
      switch ($action_id) {
        case '7': // isuma ftp uploads
          return mm_isuma_ftp_node_save($file, $configuration);
        break;
        case '9': // isuma ftp uploads
          return mm_isuma_livetv_node_save($file, $configuration);
        break;
        case '10': // isuma metadata
          return mm_isuma_metadata($file, $configuration);
        break;
      }
    break;
    
    // define the processing that this module does
    case 'process':
      switch ($action_id) {
        case '1': //convert video
          return mm_isuma_video_hi($file, $configuration);
        break;
        case '2': //convert video
          return mm_isuma_video_lo($file, $configuration);
        break;
        case '3': //create thumbnail from video
           return mm_isuma_video_thumb($file, $configuration);
        break;
        case '4': //create audio
           return mm_isuma_audio($file, $configuration);
        break;
        case '5': //create ipod video
           return mm_isuma_ipod($file, $configuration);
        break;
        case '11': //create hd video
           return mm_isuma_video_hd($file, $configuration);
        break;
      }
    break;           
  }
  return;
}


/**
 * Implementation of hook_help().
 */
function mm_isuma_help($path, $arg) {
  switch ($path) {
    case 'admin/modules#description' :
      return t('Helper module for Media Mover. Allows users to convert files with Isuma presets');
  }
  return;
}


/* ************************************ */
/* conversion functions */
/* ************************************ */


/**
 * Process a file on a local server to produce the high-bandwith version.
 * Based on code by isuma.tv
 * 
 * @param $filename The path the the file to work on
 * @param $destination The directory in which to place the finished file
 * @return destination filename on success, False otherwise
 */
function ls_isuma_video_hi($filename, $destination) {
  // get the file we should be operating on
  $current_file = $filename;

  // Get file metadata
  $meta = ffmpeg_wrapper_file_data($current_file);
  $pathinfo = pathinfo($current_file);
  
  // If already FLV
  if (strpos($meta['format'], 'flv')!==FALSE) {
  
    // Construct the file name
    // We need to strip off the last file extension and append the conversion
    // type on the end. Also, replace any spaces in the file name.
    $output_file = ls_replace_file_extension( basename( $current_file ), "flv"  );
   
    //Warn if output file exists, but overwrite it
    $output_file = "$destination/$output_file";
    if( file_exists( $output_file ) ){
      trigger_error( "File $output_file already exists. Overwriting.", E_USER_WARNING );
    }
  
    copy($current_file, $output_file);
    
  } else {
  
    // Construct the file name
    // We need to strip off the last file extension and append the conversion
    // type on the end. Also, replace any spaces in the file name.
    $output_file = ls_replace_file_extension( basename( $current_file ), "flv"  );
    
    //Warn about existing file, but overwrite it
    $output_file = "$destination/$output_file";
    if( file_exists( $output_file ) ){
      trigger_error( "File $output_file already exists. Overwriting.", E_USER_WARNING );
    }
  
    // create ffmpeg command 
    $size = mm_isuma_resize(640, 480, $meta);
    if (!$size) {
      $command = "-y -i " . escapeshellarg($current_file) . " -f flv -vcodec libx264 -vpre default -b 1000k -acodec libfaac -ab 128k -ar 44100 -ac 2 " . escapeshellarg($output_file);
    } else {
      $command = "-y -i " . escapeshellarg($current_file) . " -f flv -vcodec libx264 -vpre default -b 1000k -acodec libfaac -ab 128k -ar 44100 -ac 2 " . $size . " " . escapeshellarg($output_file);
    }
    
    // run ffmpeg
    if (! $output = ffmpeg_wrapper_run_command($command)) {
      trigger_error("FFMPEG failure: $output on file $file with command $command", E_USER_WARNING);
      return false;
    }
    
    // successful convert, make a note in the log
    if( DEBUG > 0 ){
     trigger_error("Isuma local server converted this file: $output_file",  E_USER_NOTICE);
    }

  }

  // return the completed file
  return $output_file;
}


/**
 * Take a video file on a local server and transcode it to low-bandwidth format
 * Based on media mover scripts by isuma.tv
 * @params $filename The source file path and name
 * @params $destination The directory where the resulting file will be placed
 * @return the file path on success, False on failure
 */

function ls_isuma_video_lo($filename, $destination) {
  // get the file we should be opperating on
  $current_file = $filename;

  // Construct the file name
  // We need to strip off the last file extension and append the conversion
  // type on the end. Also, replace any spaces in the file name.
  $output_file = ls_replace_file_extension( basename( $current_file ), "flv"  );
  
  //If output file exists, warn, but overwrite
  $output_file = "$destination/$output_file";
  if( file_exists( $output_file ) ){
    trigger_error( "File $output_file already exists. Overwriting.", E_USER_WARNING );
  }
  
  // Get file metadata
  $meta = ffmpeg_wrapper_file_data($current_file);
  
    // creat ffmpeg command 
    $size = mm_isuma_resize(640, 360, $meta);
    if (!$size) {
      $command = "-y -i " . escapeshellarg($current_file) . " -f flv -vcodec libx264 -vpre default -b 100k -r 15 -acodec libfaac -ab 64k -ar 44100 -ac 2 " . escapeshellarg($output_file);
    } else {
      $command = "-y -i " . escapeshellarg($current_file) . " -f flv -vcodec libx264 -vpre default -b 100k -r 15 -acodec libfaac -ab 64k -ar 44100 -ac 2 " . $size . " " . escapeshellarg($output_file);
    }
    
    // run ffmpeg
    if (! $output = ffmpeg_wrapper_run_command($command)) {
      trigger_error("FFMPEG failure: $output on file $file with command $command", E_USER_WARNING);
      return false;
    }
 
    // successful convert, make a note in the log
    if( DEBUG > 0 ){
     trigger_error("Isuma local server converted this file: $output_file",  E_USER_NOTICE);
    }

  // return the completed file
  return $output_file;
}

function ls_isuma_video_hd($filename, $destination) {
  // get the file we should be opperating on
  $current_file = $filename; 
  
  // Construct the file name
  // We need to strip off the last file extension and append the conversion
  // type on the end. Also, replace any spaces in the file name.
  $output_file = ls_replace_file_extension( basename( $current_file ), "flv"  );

  //see if the file already exists at the destination
  //If it does, warn, but overwrite it
  $output_file = "$destination/$output_file";
  if( file_exists( $output_file ) ){
    trigger_error( "File $output_file already exists. Overwriting.", E_USER_WARNING );
  }  
  // Get file metadata
  $meta = ffmpeg_wrapper_file_data($current_file);

  if( DEBUG ) {
    trigger_error( print_r( $meta, true ), E_USER_NOTICE );
  }
  
  if ($meta['video']['s']) {
    $size = explode('x', $meta['video']['s']);
  
    if (($size[0] > 1279) OR ($size[1] > 719)) {
    
      // create ffmpeg command 
      $size = mm_isuma_resize(1280, 720, $meta);
      if (!$size) {
        $command = "-y -i " . escapeshellarg($current_file) . " -f flv -vcodec libx264 -vpre default -b 4000k -acodec libfaac -ab 128k -ar 44100 -ac 2 " . escapeshellarg($output_file);
      } else {
        $command = "-y -i " . escapeshellarg($current_file) . " -f flv -vcodec libx264 -vpre default -b 4000k -acodec libfaac -ab 128k -ar 44100 -ac 2 " . $size . " " . escapeshellarg($output_file);
      }
      
      // run ffmpeg
      if (! $output = ffmpeg_wrapper_run_command($command)) {
        trigger_error("FFMPEG failure: $output on file $filename with command $command", E_USER_WARNING);
        return false;
      }
   
      // successful convert, make a note in the log
      if( DEBUG > 0 ){
       trigger_error("Isuma local server converted this file: $output_file",  E_USER_NOTICE);
      }
      
    } else {
      // filename indicates absence of hd720 file
      $output_file = ISUMA_ROOT . "media/video/hd/QsvZXZOf4xW2qdQvyFYx.flv";
    }
    
  }else{
    // filename indicates absence of hd720 file
    $output_file = ISUMA_ROOT . "media/video/hd/QsvZXZOf4xW2qdQvyFYx.flv";
  }
  
  // return the completed file
  return $output_file;
}

function mm_isuma_video_thumb(&$file, $configuration) {
  // get the file we should be opperating on
  $current_file = media_mover_api_config_current_file($file);

  // first error check, make sure that we can decode  this kind of file
  /*
  if (! ffmpeg_wrapper_can_decode($current_file)) {
    $message = 'FFmpeg can not decode this file: %file !link';
    $variables = array(
      '%file' => $current_file,
      '!link' => l(t('media mover file'), 'admin/media_mover/file/edit/'. $file['mmfid'])
    );

    $link = l("node ". $file['nid'],  "node/". $file['nid'] ."/edit");
    watchdog('MM FFmpeg', $message, $variables, WATCHDOG_ERROR, $link);
    return false;
  }
  */
  
  // Construct the file name
  // We need to strip off the last file extension and append the conversion
  // type on the end. Also, replace any spaces in the file name.
  $pattern = array('/(.*)\..*$/', '/(\s)/');
  $replace = array('$1.jpg', '_');
  $output_file = preg_replace($pattern, $replace, basename($current_file));
  if (substr($output_file, -4, 4) != '.jpg') {
    $output_file .= '.jpg';
  }
  // create the destination path
  $destination = media_mover_api_dir_path('isuma/'. $configuration['cid']);
  // now make sure that this file does not already exist at the destination
  $output_file = file_destination("$destination/$output_file", FILE_EXISTS_RENAME);
  
  // Get file metadata
  $meta = ffmpeg_wrapper_file_data($current_file);
  
    // create ffmpeg command 
    $size = mm_isuma_resize(640, 480, $meta);
    $seek = intval($meta['duration'] / 2);
    if ($seek > 120) { $seek = 120; }
    if (!$size) {
      $command = "-i " . escapeshellarg($current_file) . " -y -vframes 1 -ss " . $seek . " -an -vcodec mjpeg -f rawvideo " . escapeshellarg($output_file);
    } else {
      $command = "-i " . escapeshellarg($current_file) . " -y -vframes 1 -ss " . $seek . " -an -vcodec mjpeg -f rawvideo " . $size . " " . escapeshellarg($output_file);
    }
    
    // run ffmpeg
    if (! $output = ffmpeg_wrapper_run_command($command)) {
      mm_ffmpeg_error_alert($output, $file, $command);
      return false;
    }
 
    // successful convert, make a note in the log
    $message = 'MM Isuma converted this file: !file' ;
    $variables = array('!file' => $output_file);
    $link = l('node '. $file['nid'],  'node/'. $file['nid'] .'/edit');
    watchdog('MM FFmpeg', $message, $variables, WATCHDOG_NOTICE, $link);
    

  // run media mover's file perms on the completed file
  media_mover_api_set_file_perm($output_file);  
  
  // can we get the mime type for this?
  if (function_exists('mimedetect_mime')) {
    $mime = mimedetect_mime($output_file);
    $file['data']['file']['mime'] = $mime; 
  }
   
  // return the completed file
  return $output_file;
}


function mm_isuma_audio(&$file, $configuration) {
  // get the file we should be opperating on
  $current_file = media_mover_api_config_current_file($file);
  
  // Construct the file name
  // We need to strip off the last file extension and append the conversion
  // type on the end. Also, replace any spaces in the file name.
  $pattern = array('/(.*)\..*$/', '/(\s)/');
  $replace = array('$1.mp3', '_');
  $output_file = preg_replace($pattern, $replace, basename($current_file));
  // create the destination path
  $destination = media_mover_api_dir_path('isuma/'. $configuration['cid']);
  // now make sure that this file does not already exist at the destination
  $output_file = file_destination("$destination/$output_file", FILE_EXISTS_RENAME);
  
  // Get file metadata
  $meta = ffmpeg_wrapper_file_data($current_file);

  // skip conversion if already mp3
  if (strpos('mp3', $meta['format'])===FALSE) {
  
    // create ffmpeg command 
      $command = "-i " . escapeshellarg($current_file) . " -f mp3 -ar 44100 -ab 128k -ac 2 " . escapeshellarg($output_file);

    // run ffmpeg
    if (! $output = ffmpeg_wrapper_run_command($command)) {
      mm_ffmpeg_error_alert($output, $file, $command);
      return false;
    }
 
    // successful convert, make a note in the log
    $message = 'MM Isuma converted this file: !file' ;
    $variables = array('!file' => $output_file);
    $link = l('node '. $file['nid'],  'node/'. $file['nid'] .'/edit');
    watchdog('MM FFmpeg', $message, $variables, WATCHDOG_NOTICE, $link);
    
  } else {
    copy($current_file, $output_file);
  }

  // run media mover's file perms on the completed file
  media_mover_api_set_file_perm($output_file);  
  
  // can we get the mime type for this?
  if (function_exists('mimedetect_mime')) {
    $mime = mimedetect_mime($output_file);
    $file['data']['file']['mime'] = $mime; 
  }
   
  // return the completed file
  return $output_file;
}

/** Convert a file on a local server to iPod format.
 * 
 * Based on MediaMover scripts written by isuma.tv
 * @param $filename The full path to the file on the local filesystem.
 * @param $destination the location to place the output file
 * @return filename if conversion succeeds, False otherwise.
 */
function ls_isuma_ipod($filename, $destination ) {
  // get the file we should be opperating on
  $current_file = $filename;
  
  // Construct the file name
  // We need to strip off the last file extension and append the conversion
  // type on the end. Also, replace any spaces in the file name.
  $output_file = ls_replace_file_extension( basename( $current_file ), "mp4"  );
  
  //If destination file exists, warn, but overwrite
  $output_file = "$destination/$output_file";
  if( file_exists( $output_file ) ){
    trigger_error( "File $output_file already exists. Overwriting.", E_USER_WARNING );
  }
  
  // Get file metadata
  $meta = ffmpeg_wrapper_file_data($current_file);
//  dpm($meta);
  
    // creat ffmpeg command 
    $size = mm_isuma_resize(480, 360, $meta);
    if (!$size) {
      $command = "-y -i " . escapeshellarg($current_file) . " -f mp4 -vcodec libx264 -vpre default -vpre ipod640 -b 500k -acodec libfaac -ab 64k -ar 44100 -ac 2 " . escapeshellarg($output_file);
    } else {
      $command = "-y -i " . escapeshellarg($current_file) . " -f mp4 -vcodec libx264 -vpre default -vpre ipod640 -b 500k -acodec libfaac -ab 64k -ar 44100 -ac 2 " . $size . " " . escapeshellarg($output_file);
    }
    
    // run ffmpeg
    if (! $output = ffmpeg_wrapper_run_command($command)) {
      trigger_error("FFMPEG failure: $output on file $file with command $command", E_USER_WARNING);
      return false;
    }
    
    // make it streamable
    
    $temp_file = $output_file . ".tmp";
    rename($output_file, $temp_file );
    $command = '/usr/bin/qt-faststart ' . escapeshellarg($temp_file) . ' ' . escapeshellarg($output_file);
    exec($command);
    unlink($temp_file);

    //Make note of the successful conversion in the logs
    if( DEBUG > 0 ){
     trigger_error("Isuma local server converted this file: $output_file",  E_USER_NOTICE);
    }

  // return the completed file
  return $output_file;
}


function mm_isuma_ftp_harvest ($configuration) {

  $filetypes = 'avi|mpeg|mpg|mov|rm|flv|wmv|3gp|mp4|dir|dcr|ogg|f4v|m4v';

  // make sure that we have both lower and upper case versions of the file
  // extensions. file_scan_directory uses ereg() for the mask which
  // means we have to pass both upper and lower case version
  $filetypes = strtoupper($filetypes) .'|'. strtolower($filetypes);

  // now add the regex to the pattern
  $pattern = ".*($filetypes)$";
  // get files restricted by file extension

  foreach (file_scan_directory('/persistent/www/isumatv/sites/default/files/video-ftp', '^[0-9]*$', array('.', '..'), 0, FALSE) as $group) {
    foreach (file_scan_directory($group->filename, '^[0-9]*$', array('.', '..'), 0, FALSE) as $user) {
      foreach (file_scan_directory($user->filename, $pattern, array('.', '..'), 0, TRUE) as $file) {
        // isuma_process_file($file->filename, (int) $group->basename, (int) $user->basename);
        
        // check the returned files

        // now we need to see if we have this file in the database. Note that in
        // this instance, we are not checking dates on files vs. the database
        $result = db_fetch_array(db_query('SELECT mmfid FROM {media_mover_files} WHERE cid = %d AND harvest_file = "%s"', $configuration['cid'], $file->filename));
        if (! $result) {
          $node['uid'] = $user->basename;
          $node['og_groups'] = array($group->basename => $group->basename);
          $node['title'] = substr($file->basename, 0, strlen($file->basename)-4);
          
          //transliterate filename
          $filename = $file->filename;
          $newfilename = basename($filename);
          require_once(drupal_get_path('module', 'transliteration') .'/transliteration.inc');
          $newfilename = transliteration_clean_filename($newfilename);
          file_move($filename, '/persistent/www/isumatv/sites/default/files/video-ftp/video/' . $newfilename);
          $file->filename = $filename;
  
          $output = ffmpeg_wrapper_file_data($file->filename);
          if ($output['format'] != 'na') {
            $files[] = array('harvest_file' => $file->filename, 'node' => $node);
          }
        }
	   }
    }
  }


  // if we have files, return them
  if (count($files)) {
    return $files;
  }
}

function mm_isuma_ftp_node_save(&$file, $configuration) {
  // get the path to the file to operate on. We need this
  // to pass out of the function
  $file_path = media_mover_api_config_current_file($file);

  // set node type
  $node = $file['data']['node'];
  $node['type'] = 'video';
  $node['status'] = 1;

  // Now build the node author
  $account = user_load($node['uid']);

  // attach the user data
  $node['name'] = $account->name;

  $node['comment'] = 2;
  $node['og_public'] = TRUE;
  
  $node['field_url'][0]['value'] = 1;
  $node['field_embed'][0]['value'] = 1;
  
  // save the new node
  $node = node_submit($node);
  node_save($node);
  
  // save file in files table
  db_query("INSERT INTO {files} (uid, filename, filepath, status, timestamp, filemime, filesize) VALUES (%d, '%s', '%s', %d, %d, '%s', %d)",
    $node->uid, basename($file_path), substr($file_path, 24), 1, time(), file_get_mimetype($file_path), filesize($file_path)
  );
  $fid = db_last_insert_id('file', 'fid');
  
  // save file in video content table
  db_query('UPDATE {content_type_video} SET field_video_fid = %d WHERE nid = %d', $fid, $node->nid );
    
  // append the $nid to the $file for easy retreval
  $file['nid'] = $node->nid;
  // add the node data to the file object
  $file['node'] = $node;
  // make a log entry
  watchdog('Media Mover Node', 'Created node: !node', array('!node' => l(t($node->title),'node/'. $node->nid)));

  // return the path on the file saved to the $node
  return $file_path;
}


function mm_isuma_livetv_harvest ($configuration) {

  $filetypes = 'flv';

  // make sure that we have both lower and upper case versions of the file
  // extensions. file_scan_directory uses ereg() for the mask which
  // means we have to pass both upper and lower case version
  $filetypes = strtoupper($filetypes) .'|'. strtolower($filetypes);

  // now add the regex to the pattern
  $pattern = ".*($filetypes)$";
  // get files restricted by file extension
  
  $nodes = array();

  foreach (file_scan_directory('/persistent/www/isumatv/sites/default/files/video-live', $pattern) as $file) {
    // now we need to see if we have this file in the database. Note that in
    // this instance, we are not checking dates on files vs. the database
    $result = db_fetch_array(db_query('SELECT mmfid FROM {media_mover_files} WHERE cid = %d AND harvest_file = "%s"', $configuration['cid'], $file->filename));
    if (! $result) {
      $stream = substr($file->basename, 0, 10);
      $result = db_fetch_array(db_query('SELECT nid FROM {content_type_livetv} WHERE field_livetv_stream = "%s"', $stream));
      if ($result) {
        $node = node_load($result['nid']);
        if ($node->field_livearchive[0]['value'] == 'stopped') {
          $nodes[] = $node;
          $filename = $file->filename;
          
          $pattern = array('/(.*)\..*$/', '/(\s)/');
          $replace = array('$1.flv', '_');
          $output_file = preg_replace($pattern, $replace, basename($filename));
          // create the destination path
          $destination = "/persistent/www/isumatv/sites/default/files/video-live-archive";
          // now make sure that this file does not already exist at the destination
          $output_file = file_destination("$destination/$output_file", FILE_EXISTS_RENAME);
          
//          $command = "-i " . escapeshellarg($filename) . " -vcodec copy -acodec libfaac -ab 96k -async 22050 " . escapeshellarg($output_file);
          $command = "-i " . escapeshellarg($filename) . " -vcodec copy -acodec copy " . escapeshellarg($output_file);
          ffmpeg_wrapper_run_command($command);
          file_delete($filename);

          $files[] = array('harvest_file' => $output_file, 'node' => $node);
          
        }
      }
    }
  }
  
  foreach ($nodes as $node) {
    $node->field_livearchive[0]['value'] = 'broadcast';
    node_save($node);
  }

  // if we have files, return them
  if (count($files)) {
    return $files;
  }
}

function mm_isuma_livetv_node_save(&$file, $configuration) {
  // get the path to the file to operate on. We need this
  // to pass out of the function
  $file_path = media_mover_api_config_current_file($file);

  // set node type
  $original_node = $file['data']['node'];
  
  $node['type'] = 'video';
  $node['status'] = 1;
  $node['comment'] = 2;
  $node['og_public'] = TRUE;
  $node['uid'] = $original_node->uid;
  $node['title'] = $original_node->title;
  $node['body'] = $original_node->body;
  $node['name'] = $original_node->name;
  $node['og_groups'] = $original_node->og_groups;
  $node['taxonomy'] = $original_node->taxonomy;
  $node['field_url'][0]['value'] = 1;
  $node['field_embed'][0]['value'] = 1;
  
  // save the new node
  $node = node_submit($node);
  node_save($node);
  
  // save file in files table
  db_query("INSERT INTO {files} (uid, filename, filepath, status, timestamp, filemime, filesize) VALUES (%d, '%s', '%s', %d, %d, '%s', %d)",
    $node->uid, basename($file_path), substr($file_path, 24), 1, time(), file_get_mimetype($file_path), filesize($file_path)
  );
  $fid = db_last_insert_id('file', 'fid');
  
  // save file in video content table
  db_query('UPDATE {content_type_video} SET field_video_fid = %d WHERE nid = %d', $fid, $node->nid );
    
  // append the $nid to the $file for easy retreval
  $file['nid'] = $node->nid;
  // add the node data to the file object
  $file['node'] = $node;
  // make a log entry
  watchdog('Media Mover Node', 'Created node: !node', array('!node' => l(t($node->title),'node/'. $node->nid)));

  // return the path on the file saved to the $node
  return $file_path;
}

function mm_isuma_metadata(&$file, $configuration) {
  // get the file we should be opperating on
  $current_file = media_mover_api_config_current_file($file);
  
  // Get file metadata
  $meta = ffmpeg_wrapper_file_data($current_file);
  $size = filesize($current_file);
  $node = node_load($file['nid']);
  $node->field_filesize[0]['value'] = $node->field_filesize[0]['value'] + $size;
  $node->field_duration[0]['value'] = $node->field_duration[0]['value'] + $meta['duration'];
  node_save($node);
  
  // return the completed file
  return $current_file;
}


/**
 * Return resize code
 */
function mm_isuma_resize($width, $height, $meta) {
  if (!$meta['video']['s']) {
    return FALSE;
  }
  $size = explode('x', $meta['video']['s']);
  $width_ar =  $width / $size[0];
  $height_ar = $height / $size[1];
  if (($width_ar >= 1) and ($height_ar >= 1)) {
    return FALSE;
  }
  if ($width_ar < $height_ar) {
    $height = floor(($width * $size[1]) / $size[0]);
    if ($height % 2 != 0) {
      $height = $height - 1;
    }
  } else {
    $width = floor(($height * $size[0]) / $size[1]);
    if ($width % 2 != 0) {
      $width = $width - 1;
    }
  }
  return "-s " . $width . "x" . $height;
}

/**
 * Helper function that takes a file name and an extension, and 
 * appends the extension to the filename.
 * @param $filename The original filename. May include path components.
 * @param $extension The new extension to replace the old one. Will be appended if no extension currently exists.
 * @return the new filename
 */

function ls_replace_file_extension( $filename, $extension ){
/*  $pattern = array('/(.*)\..*$/', '/(\s)/');
  $replace = array('$1.' . $extension , '_');
  $output_file = preg_replace($pattern, $replace, basename($filename));
  $len = strlen( ".$extension" );
  if (substr($output_file, -$len, $len ) != ".$extension") {
    $output_file .= ".$extension";
  }
  */
  return $filename . "." . $extension ;
  //return $output_file;

}
